#include <TimerOne.h>

const float Alarmtemperatur = 18.0f;//°C
const int Pin_Lautsprecher = 9; //9 oder 10;
const int Pin_Messung = A1;
const double R2 = 1976.0; //Mit Multimeter gemessen
const double R_ref = 2000.0; //Wert des Thermistors bei 25°C

void setup() {
  //Bereite den Lautsprecher vor
  pinMode(Pin_Lautsprecher, OUTPUT);
  Timer1.initialize();
  Timer1.pwm(9, 0, 300);

  Serial.begin(9600);
}

double berechne_Widerstand() {
  //Spannung, die über R1 abfällt
  double U1 = analogRead(A1) / 1024.0 * 5.0;

  //Berechnung von R1 über Umformung
  //  der Spannungsteiler-Formel
  double R1 = (R2 * U1) / (5.0 - U1);
  return R1;
}

double berechne_Temperatur(double R1) {
  //Parameter des Thermistors aus dem Datenblatt
  double A, B, C, D;

  //Um welchen Faktor unterscheidet sich der gemessene Wert
  //vom Referenzwer von 2kOhm?
  double R = R1 / R_ref;

  //Fallunterscheidung: Die Berechnung variiert,
  // je nach betrachtetem Temperaturbereich
  if (R > 1.0) {
    //Fall 1: Temperatur < 25°C
    A = 3.354016e-3;
    B = 2.90967e-4;
    C = 1.632136e-6;
    D = 7.1922e-8;
  } else {
    //Fall 2: Temperatur >= 25°C
    A = 3.354016e-3;
    B = 2.933908e-4;
    C = 3.494314e-6;
    D = -7.71269e-7;
  }

  //Einsetzen der Werte in die Formel und Berechnung
  //der Temperatur
  double T = 1.0 / (A
                    + B * log(R)
                    + C * (log(R) * log(R))
                    + D * (log(R) * log(R) * log(R)));
  return T - 273.15;
}

void alarm() {
  for (int i = 0; i < 10; i++) {
    for (int j = 0; j < 3; j++) {
      Timer1.pwm(9, 510);
      delay(500);
      Timer1.pwm(9, 0);
      delay(500);
    }
    delay(1000);
  }
}

void loop() {
  double R1 = berechne_Widerstand();
  double Temperatur = berechne_Temperatur(R1);

  //Ausgabe
  Serial.print("Die Temperatur ist ");
  Serial.print(Temperatur);
  Serial.println(" Grad Celsius");

  if (Temperatur < Alarmtemperatur) {
    Serial.println("Alarm!");
    alarm();
    while(true) { }; //Stoppe
  }

  delay(1000);
}
