# -*- coding: utf-8 -*- 
#!/usr/bin/python
#------------------------------------------------------------------------   
# Robotik mit dem Raspberry Pi
# E.F.Engelhardt, Franzis Verlag, 2014 
# ----------------------------------------------------------------------- 
# Das Skript nutzt die Analog-Eingaenge des MCP3008 IC 
# und liest diese ueber SPI Bus aus
#
# joystick-analog-step2.py
# -----------------------------------------------------------------------
import spidev
# wget https://raw.github.com/doceme/py-spidev/master/setup.py
# wget https://raw.github.com/doceme/py-spidev/master/spidev_module.c
import time 
import os
import RPi.GPIO as GPIO
#  ----------------------------------------------------------------------- 
# SPI Objekt erstellen
GPIO.setmode(GPIO.BCM) 
# GPIO Mode
spi = spidev.SpiDev() 
spi.open(0,0)
#  -----------------------------------------------------------------------
# Joystick-Switch
GPIO_SW = 22 # PIN 15 /	GPIO-22 / wiring pi 3 / Eingang
GPIO.setup(GPIO_SW, GPIO.OUT)
GPIO.output(GPIO_SW, True)
GPIO.setup(GPIO_SW, GPIO.IN)
#  -----------------------------------------------------------------------
# Funktion um Daten aus dem MCP3008 IC zu lesen
# Kanalzaehlung von 0-7
def getadcChannel(channel):
  # Nur Kanal 0-7 erlaubt
  if ((channel > 7) or (channel < 0)):
                    return -1
  rdata = spi.xfer2([1,(8+channel)<<4,0])
  data = ((rdata[1]&3) << 8) + rdata[2]
  return data

# Funktion um Werte in Spannungswerte zu konvertieren
# (auf 2^10 Einheiten)
def convertVolts(data,places):
  volts = (data * 3.3) / 1023 # 3.3V Referenzspannung an MCP!
  volts = round(volts,places) # und runden
  return volts
# ----------------------------------------------
# Start Python-Skript
# ----------------------------------------------
os.system('clear')
print "[Joystick-Analog] Raw / Voltwerte..." 
# Kanalnummer auswaehlen
VRx_channel = 5
VRy_channel = 6
# max 7 !
#
# Pause zwischen Messungen 
delay = 1 # 1 Sekunde
# dezimalstellen
places = 3
try: 
# Dauer-Schleife
  while True:
    os.system('clear')
    print ('[Joystick am Rasbperry Pi...]')
    # Joystick VRx
    vrx_level = getadcChannel(VRx_channel)
    vrx_volts = convertVolts(vrx_level,places) 
    # Joystick VRy
    vry_level = getadcChannel(VRy_channel)
    vry_volts = convertVolts(vry_level,places)
    # Datenausgabe
    if not(GPIO.input(GPIO_SW)):
	print ("-----------------------------------")
	print ("[Joystick] Schalter wurde gedrckt!")
	print "Status=", (GPIO.input(GPIO_SW))
    print ("-----------------------------------")
#zero_x = 782
#zero_y = 760
        #NORMAL
    elif(vrx_level > 770 and vrx_level < 790 and vry_level > 755 and vry_level < 765):
#    if(vrx_level > 760 and vrx_level < 790 and vry_level > 740 and vry_level < 765):
        print("------------")
        print("NORMAL", "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #V
    elif(vrx_level > 790 and vrx_level < 1024 and vry_level > 845 and vry_level < 865):
        print("------------")
        print("VORWAERTS",  "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #BACK
    elif(vrx_level < 10 and vry_level > 755 and vry_level < 765):
        print("------------")
        print("RUECKWAERTS", "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #LEFT
    elif (vrx_level > 775 and vrx_level < 785 and vry_level < 10 ):
        print("------------")
        print("LINKS",  "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #RIGHT
    elif (vrx_level > 775 and vrx_level < 910 and vry_level > 865 and vry_level < 1024):
        print("------------")
        print("RECHTS",  "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #FRONT-RIGHT
    elif (vrx_level > 910 and vrx_level < 1024 and vry_level > 865 and vry_level < 1024):
        print("------------")
        print("VORWAERTS-RECHTS", "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #BACK-RIGHT
    elif (vrx_level >= 0 and vrx_level < 10 and vry_level > 765 and vry_level < 1024):
        print("------------")
        print("RUECKWAERTS-RECHTS",  "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #BACK-LEFT
    elif (vrx_level >= 0 and vrx_level < 10 and vry_level >= 0 and vry_level < 10):
        print("------------")
        print("RUECKWAERTS-LINKS", "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #FRONT-LEFT
    elif (vrx_level > 775 and vrx_level < 1024 and vry_level < 10 ):
        print("------------")
        print("VORWAERTS-LINKS", "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
  #UNKNOWN
    else:
#        print("------------")
#        print("UNBEKANNTE SPANNUNG")
        print ( "Status=", (GPIO.input(GPIO_SW)))
        print("------------")
    print("Joystick Variable X: {} ({}V)".format(vrx_level,vrx_volts))
    print("Joystick Variable Y: {} ({}V)".format(vry_level,vry_volts))
    time.sleep(delay)
except KeyboardInterrupt:
    # CTRL-C gedrueckt
    # Reset GPIO
    print "[Joystick] Messung abgebrochen."
    GPIO.cleanup()
# ----------------------- EOF  ---------------------------
