#!/usr/bin/env python
# -*- coding: utf-8 -*- 
#---------------------------------------------------------------------------------------- 
# Robotik mit dem Raspberry Pi 
# E.F.Engelhardt, Franzis Verlag, 2014 
#----------------------------------------------------------------------------------------
# i2c_lib2.py 
# mit BMP085 Funktionen Schnellzugriffsfunktionen auf i2c-Bus
# mit MRP121-Touchfunktionen getestet
# 
import smbus, os 
import time
#----------------------------------------------------------------------------------------
class i2c_device:
   @staticmethod
   def getPiRevision():
      try:
         f = open('/proc/cpuinfo','r')
         for line in f:
              if line.startswith('Revision'):
               return 1 if line.rstrip()[-1] in ['1','2'] else 2
         f.close()
      except:
         return 0
   #-------------------------------------------------------------------------------------
   @staticmethod
   def getPiI2CBusNumber():
       # Gets the I2C bus number /dev/i2c#
       return 1 if i2c_device.getPiRevision() > 1 else 0
   #-------------------------------------------------------------------------------------
   def __init__(self, addr, port=-1):
      self.addr = addr
      print "i2c-Adresse-Geraet= ", hex(self.addr)
      print "i2c-Busnummer= ", i2c_device.getPiI2CBusNumber()
      print "Raspberry-Pi Revision=", i2c_device.getPiRevision()
      self.bus = smbus.SMBus(port if port >= 0 else i2c_device.getPiI2CBusNumber())
   #-------------------------------------------------------------------------------------
   def write_cmd(self, cmd): # Befehl an Adresse
      self.bus.write_byte(self.addr, cmd)
      sleep(0.0001)
   #-------------------------------------------------------------------------------------
   def write_cmd_arg(self, cmd, data): # Befehl und Wert an Adresse
      self.bus.write_byte_data(self.addr, cmd, data)
      sleep(0.0001)
   #-------------------------------------------------------------------------------------
   def write_block_data(self, cmd, data): # Datenblock schreiben
      self.bus.write_block_data(self.addr, cmd, data)
      sleep(0.0001)
   #-------------------------------------------------------------------------------------
   def read(self): # Byte lesen
      return self.bus.read_byte(self.addr)
   #-------------------------------------------------------------------------------------
   def read_data(self, cmd): # Inhalt lesen
      return self.bus.read_byte_data(self.addr, cmd)
   #-------------------------------------------------------------------------------------
   def read_block_data(self, cmd): # Datenblock auslesen
      return self.bus.read_block_data(self.addr, cmd)
   #-------------------------------------------------------------------------------------
   def read_word_data(self, cmd): # Datenblock auslesen
      return self.bus.read_word_data(self.addr, cmd)
   #-------------------------------------------------------------------------------------
   def readWord(self, cmd): # Word-Register lesen
    # read most significant byte (msb)
     msb = self.bus.read_byte_data(self.addr, cmd)
     # read least significant byte (lsb)
     lsb = self.bus.read_byte_data(self.addr, cmd+1)
     valueW = (msb << 8) + lsb
     return valueW
   #-------------------------------------------------------------------------------------
   def readSignedWord(self, cmd): # signed word aus register lesen
     # read most significant byte debug print "hier adress", hex(self.addr)
     msb = self.bus.read_byte_data(self.addr, cmd)
     # read least significant byte
     lsb = self.bus.read_byte_data(self.addr, cmd+1)
     if (msb > 127):
       msb = msb - 256
     valueSW = (msb << 8) + lsb
     return valueSW
   #-------------------------------------------------------------------------------------
   def readRawTemperature(self): # read uncompensated temperature value
     # write 0x2E into register 0xF4 to request temperature reading
     self.bus.write_byte_data(self.addr, 0xF4, 0x2E)
     # wait 5ms
     time.sleep(0.005)
     # read two byte result from address 0xF6
     valueRT = self.readWord(0xF6)
     return valueRT
   #-------------------------------------------------------------------------------------
   def readRawPreassure(self, mode): # read uncompensated pressure value
     self.mode = mode
     # write 0x34+(BMP085_OVERSAMPLING_SETTING<<6) into register 0xF4
	 # to request pressure reading with oversampling setting
     self.bus.write_byte_data(self.addr, 0xF4, 0x34 + (self.mode << 6))
     # wait for conversion, delay time depends on oversampling setting
     delay = (2 + (3 << self.mode)) / 1000.0
     time.sleep(delay)
     # read three byte result from address 0xF6 (0xF6 = MSB, 0xF7 = LSB, 0xF8 = XLSB)
     msb = self.bus.read_byte_data(self.addr, 0xF6)
     lsb = self.bus.read_byte_data(self.addr, 0xF7)
     xlsb = self.bus.read_byte_data(self.addr, 0xF8)
     # calculate uncompensated pressure value
     valueRP = ((msb << 16) + (lsb << 8) + xlsb) >> (8 - self.mode)
     return valueRP
# --------------------------------- EOF --------------------------------------------
