# -*- coding: utf-8 -*- 
#!/usr/bin/python 
#------------------------------------------------------------------------
# Robotik mit dem Raspberry Pi
# E.F.Engelhardt, Franzis Verlag, 2014 MPR121 Register 
# -----------------------------------------------------------------------
# Bibliotheks-Datei mpr121.py
# -----------------------------------------------------------------------
# Basiert wesentlich auf Arduino-Code
# http://bildr.org/2011/05/mpr121_arduino/
# und auf Datenblatt
# https://www.sparkfun.com/datasheets/Components/MPR121.pdf
# 
import i2c_lib2
import smbus
# I2C Bus 
# 0: Raspberry Pi Model B Rev1
# 1: Raspberry Pi Model B Rev2
I2C_MPR121_ADDRESS = 0x5a
# ----------------------------------------------------------------------- 
class MPR121():
   def __init__(self):
      self.mpr_device = i2c_lib2.i2c_device(I2C_MPR121_ADDRESS)
      self.setConst()

   def setConst(self): # Konstanten festlegen
      print " [ MPR121 ] Konstanten werden definiert..."
      self.MHD_R = 0x2B
      self.NHD_R = 0x2C
      self.NCL_R = 0x2D
      self.FDL_R = 0x2E
      self.MHD_F = 0x2F
      self.NHD_F = 0x30
      self.NCL_F = 0x31
      self.FDL_F = 0x32
      self.ELE0_T = 0x41
      self.ELE0_R = 0x42
      self.ELE1_T = 0x43
      self.ELE1_R = 0x44
      self.ELE2_T = 0x45
      self.ELE2_R = 0x46
      self.ELE3_T = 0x47
      self.ELE3_R = 0x48
      self.ELE4_T = 0x49
      self.ELE4_R = 0x4A
      self.ELE5_T = 0x4B
      self.ELE5_R = 0x4C
      self.ELE6_T = 0x4D
      self.ELE6_R = 0x4E
      self.ELE7_T = 0x4F
      self.ELE7_R = 0x50
      self.ELE8_T = 0x51
      self.ELE8_R = 0x52
      self.ELE9_T = 0x53
      self.ELE9_R = 0x54
      self.ELE10_T = 0x55
      self.ELE10_R = 0x56
      self.ELE11_T = 0x57
      self.ELE11_R = 0x58
      self.FIL_CFG = 0x5D
      self.ELE_CFG = 0x5E
      self.GPIO_CTRL0 = 0x73
      self.GPIO_CTRL1 = 0x74
      self.GPIO_DATA = 0x75
      self.GPIO_DIR = 0x76
      self.GPIO_EN = 0x77
      self.GPIO_SET = 0x78
      self.GPIO_CLEAR = 0x79
      self.GPIO_TOGGLE = 0x7A
      self.ATO_CFG0 = 0x7B
      self.ATO_CFGU = 0x7D
      self.ATO_CFGL = 0x7E
      self.ATO_CFGT = 0x7F
     # Global Constants
     # touch and release thresholds for each electrode
      self.TOU_THRESH = 0x06
      self.REL_THRESH = 0x0A

   def readData(self): #.address=I2C_MPR121_ADDRESS):
      #MSB = self.mpr_device.read_byte_data(address, 0x00)
      #LSB = self.mpr_device.read_byte_data(address, 0x01)
      touchData = self.mpr_device.read_word_data(0x01) 
      return touchData;

   def setup(self): #.address=I2C_MPR121_ADDRESS):
      self.mpr_device.write_byte_data(self, self.ELE_CFG, 0x00)

      # Section A - Controls filtering when data is > baseline.
      self.mpr_device.write_byte_data(self, self.MHD_R, 0x01)
      self.mpr_device.write_byte_data(self, self.NHD_R, 0x01)
      self.mpr_device.write_byte_data(self, self.NCL_R, 0x00)
      self.mpr_device.write_byte_data(self, self.FDL_R, 0x00)
  
     # Section B - Controls filtering when data is < baseline.
      self.mpr_device.write_byte_data(self, self.MHD_F, 0x01)
      self.mpr_device.write_byte_data(self, self.NHD_F, 0x01)
      self.mpr_device.write_byte_data(self, self.NCL_F, 0xFF)
      self.mpr_device.write_byte_data(self, self.FDL_F, 0x02)   
   
      #Section C - Sets touch and release thresholds for each electrode
      self.mpr_device.write_byte_data(self, self.ELE0_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE0_R, REL_THRESH)
   
      self.mpr_device.write_byte_data(self, self.ELE1_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE1_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE2_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE2_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE3_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE3_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE4_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE4_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE5_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE5_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE6_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE6_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE7_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE7_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE8_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE8_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE9_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE9_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE10_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE10_R, REL_THRESH)

      self.mpr_device.write_byte_data(self, self.ELE11_T, TOU_THRESH)
      self.mpr_device.write_byte_data(self, self.ELE11_R, REL_THRESH)   

      # Section D
      # Set the Filter Configuration
      # Set ESI2
      self.mpr_device.write_byte_data(self, self.FIL_CFG, 0x04)

      # Section E
      # Electrode Configuration
      # Set ELE_CFG to 0x00 to return to standby mode
      self.mpr_device.write_byte_data(self, self.ELE_CFG, 0x0C)  # Enable   
