// Datei: pretzelpir.ino
/*
Hinweis für Leser - die nachstehenden Zeilen sind im Code anzupassen:
#define SSID "WLANDAHOAM"
#define PASSWORD "WPAPWLANDAHOAM"
#define DEVID_PIR "12345678"
#define RPI_IP "192.168.123.49"
#define MAILDELTA 3000
const int threshold = 100; 
const char rpiPost[] PROGMEM = {  "GET *URL* HTTP/1.1\nHost: 192.168.123.49\nAuthorization: Basic YWRtaW46cHdhZG1pbg==\nConnection: close\nContent-Type: application/x-www-form-urlencoded\nContent-Length: *LEN*\n\n\n\0"};
 */
#define SSID "WLANDAHOAM"
#define PASSWORD "WPAPWLANDAHOAM"
#define DEVID_PIR "12345678"
#define RPI_IP "192.168.123.49"
#define MAILDELTA 3000

#define LED_WLAN 13
#define PIRSENSOR 5 
#define LEDPIN 3
#define DEBUG true

const int threshold = 100;   // Empfindlichkeit Sensor korrigieren 
int sensorReading = 0;      
int ledState = LOW;         
int sensorCounter = 0;

#include <SoftwareSerial.h>
SoftwareSerial esp8266(11, 12); // RX, TX

// OHNE AUTH
// const char rpiPost[] PROGMEM = {  "POST *URL* HTTP/1.1\nHost: 192.168.123.49\nConnection: close\nContent-Type: application/x-www-form-urlencoded\nContent-Length: *LEN*\n\n\n\0"};
// MIT AUTH
  /*
    - username = admin
    - password = admin
   ...> Terminal: echo -n admin:pwadmin | base64
   = YWRtaW46cHdhZG1pbg==
*/
const char rpiPost[] PROGMEM = {  "GET *URL* HTTP/1.1\nHost: 192.168.123.49\nAuthorization: Basic YWRtaW46cHdhZG1pbg==\nConnection: close\nContent-Type: application/x-www-form-urlencoded\nContent-Length: *LEN*\n\n\n\0"};
//----------------------------------------------------------------------------
void setup(){
  Serial.begin(115000);
  esp8266.begin(19200);
  pinMode(LEDPIN, OUTPUT);   
  if (!espConfig()) serialDebug();
  else debug("[ESP8266] Config OK");

  if (sendCom("AT+PING=\"www.franzis.de\"", "OK"))  {
    Serial.println("[ESP8266] Ping OK");
    digitalWrite(13, HIGH);
  }  else  {
    Serial.println("[ESP8266] Ping Fehler");
  }
}
//----------------------------------------------------------------------------
void loop(){
  sensorReading = analogRead(PIRSENSOR);    
  int MESSWERTSENSOR_0_1 = 0;
  if (sensorReading >= threshold) {
    ledState = !ledState;   
    digitalWrite(LEDPIN, ledState);
    sensorCounter++;    
    Serial.print("\n");
    Serial.print(sensorCounter);
    Serial.println(" Bewegung!");    
    MESSWERTSENSOR_0_1 = 1; // BEWEGUNG
    if (sendRpiPost(DEVID_PIR, MESSWERTSENSOR_0_1)){ 
        debug("[ESP8266] Update Send");
        Serial.print("[ESP8266] Update - Wert: " + String(MESSWERTSENSOR_0_1));
        Serial.print(" von Sensor ");
        Serial.print(DEVID_PIR);
        Serial.println(" wurde auf Server uebertragen.");
        delay(MAILDELTA);  
        }        else {
          debug("[ESP8266] Sensor hat Bewegung erfasst, Benachrichtigung nicht moeglich.");          
          Serial.print("[ESP8266] Update - Wert: " + String(MESSWERTSENSOR_0_1));
          Serial.print(" von Sensor ");         
          Serial.println(DEVID_PIR);
        } 
  }
  delay(1000);  
}
//-----------------------------------------RPI-Server ------------------------------------
boolean sendRpiPost(String dev_id, int value){
  boolean success = true;
  String Host = RPI_IP; 
  String  Subpage = "/pretzelpir.php?dev="+dev_id+"&status="+value;
  success &= sendCom("AT+CIPSTART=\"TCP\",\"" + Host + "\",80", "OK");
  if (success){
  String urlBuffer;
  for (int i = 0; i <= sizeof(rpiPost); i++)   {
    char myChar = pgm_read_byte_near(rpiPost + i);
    urlBuffer += myChar;
  }
  urlBuffer.replace("*URL*", Subpage);
  urlBuffer.replace("*APPEND*", String("dev="+dev_id+"&status="+value));
  urlBuffer.replace("*LEN*", String(urlBuffer.length()));
  if (sendCom("AT+CIPSEND=" + String(urlBuffer.length() + 2), ">"))
    {
      esp8266.println(urlBuffer);
      // Serial.println(urlBuffer);
      esp8266.find("SEND OK");
      if (!esp8266.find("CLOSED")) success &= sendCom("AT+CIPCLOSE", "OK");
      }   else  {
        // Serial.print("FHELER!!!");
        success = false;
     }
   } 
   return success;
}
//-----------------------------------------Config ESP8266------------------------------------
boolean espConfig(){
  boolean success = true;
  esp8266.setTimeout(5000);
  success &= sendCom("AT+RST", "ready");
  esp8266.setTimeout(1000);
  if (configStation(SSID, PASSWORD)) {
    success &= true;
    debug("[ESP8266] WLAN Connected");
    debug("[ESP8266] IP :");
    debug(sendCom("AT+CIFSR"));
  }
  else
  {
    success &= false;
  }
  //shorter Timeout for faster wrong UPD-Comands handling
  success &= sendCom("AT+CIPMUX=0", "OK");
  success &= sendCom("AT+CIPMODE=0", "OK");
  return success;
}
//-----------------------------------------
boolean configStation(String vSSID, String vPASSWORT){
  boolean success = true;
  success &= (sendCom("AT+CWMODE=1", "OK"));
  esp8266.setTimeout(20000);
  success &= (sendCom("AT+CWJAP=\"" + String(vSSID) + "\",\"" + String(vPASSWORT) + "\"", "OK"));
  esp8266.setTimeout(1000);
  return success;
}
//-----------------------------------------------Controll ESP-----------------------------------------------------
boolean sendCom(String command, char respond[]){
  esp8266.println(command);
  if (esp8266.findUntil(respond, "ERROR"))  {
    return true;
  }  else  {
    debug("[ESP8266] ESP SEND ERROR: " + command);
    return false;
  }
}
//-----------------------------------------
String sendCom(String command){
  esp8266.println(command);
  return esp8266.readString();
}
//-------------------------------------------------Debug Functions------------------------------------------------------
void serialDebug() {
  while (true)  {
    if (esp8266.available())
      Serial.write(esp8266.read());
    if (Serial.available())
      esp8266.write(Serial.read());
  }
}
//-----------------------------------------
void debug(String Msg){
  if (DEBUG)  {
    Serial.println(Msg);
  }
}